#include "EventDescriptionBoard.h"
#include "Event.h"
#include "Event.hpp"

namespace Profiler
{
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint32 EventDescriptionBoard::Register(EventDescription& description)
{ 
	CRITICAL_SECTION(lock)
	board.push_back(&description);
	return board.size() - 1;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
EventDescriptionBoard& EventDescriptionBoard::Get()
{ 
	return instance;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void EventDescriptionBoard::SetSamplingFlag( int index, bool flag )
{
	BRO_VERIFY(index < (int)board.size(), "Invalid EventDescription index", return);

	if (index < 0)
	{
		for each (EventDescription* desc in board)
			desc->isSampling = flag;
	} else
	{
		board[index]->isSampling = flag;
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool EventDescriptionBoard::HasSamplingEvents() const
{
	for each (const EventDescription* desc in board)
		if (desc->isSampling)
			return true;

	return false;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const std::vector<EventDescription*>& EventDescriptionBoard::GetEvents() const
{
	return board;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
OutputDataStream& operator << ( OutputDataStream& stream, const EventDescriptionBoard& ob)
{
	const std::vector<EventDescription*>& events = ob.GetEvents();

	stream << events.size();

	for each ( const EventDescription *desc in events )
		stream << *desc;

	return stream;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PROFILER_API uint32 RegisterEventDescription(EventDescription& description)
{
	return EventDescriptionBoard::Get().Register(description);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Profiler::EventDescriptionBoard EventDescriptionBoard::instance;
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}